@@ -397,6 +397,15 @@ public:
              const Http::ParameterMap& parameters = Http::ParameterMap(),
              const Http::UploadedFileMap& files = Http::UploadedFileMap());
 
+  /*! \brief Writes the resource into a unsigned char vector.
+   *
+   *  This is a convenience method to serialize the resource into a
+   *  vector of unsigned char. This is especially useful if you want
+   *  to convert the resource into a WMemoryResource (when overriding
+   *  botResource() for instance).
+   */
+  std::vector<unsigned char> writeToMemory();
+
   /*! \brief Handles a request.
    *
    * Reimplement this method so that a proper response is generated
@@ -472,10 +481,89 @@ public:
    */
   bool takesUpdateLock() const { return takesUpdateLock_; }
 
+  /*! \brief Sets whether this resource can be automatically removed.
+   *
+   * If enabled on a public resource, the resource will be counted as
+   * an auto-removable resource.
+   *
+   * The total number of auto-removable resources is limited by the
+   * configuration option
+   * <code>max-auto-removable-public-resources</code>. If the number of
+   * auto-removable resources exceeds this limit, the oldest
+   * auto-removable resource will be removed from the public resources.
+   *
+   * By default, this is set to false. Except for
+   * WSelfDeletingResource for which it is true.
+   *
+   * \sa botResource()
+   */
+  void setAllowAutoRemoval(bool allow);
+
+  /*! \brief Returns whether this resource can be automatically removed.
+   *
+   * \sa setAllowAutoRemoval()
+   */
+  bool allowAutoRemoval() const { return allowAutoRemoval_; }
+
   unsigned long version() const;
 
   void incrementVersion();
 
+  /*! \brief Sets the bot resource ID.
+   *
+   * This id will be appended to the resource path of the bot resource.
+   *
+   * This is useful to avoid storing the same resources multiple times
+   * in memory since only one resource can be served at any given path.
+   *
+   * By default, returns WResource::id().
+   *
+   * \warning This means that every time the resource is exposed in a
+   *          bot session, the bot resource created will be added to
+   *          the server's  public resources. In that case, you should
+   *          ensure that the resource is removed from the public
+   *          resources once it is no longer needed.
+   *
+   * \sa botResource(), WSelfDeletingResource
+   */
+  void setBotResourceId(const std::string& id);
+
+  /*! \brief Returns the bot resource ID.
+   *
+   * \sa setBotResourceId()
+   */
+  std::string botResourceId() const { return botResourceId_; }
+
+  /*! \brief Resource made public for bot sessions.
+   *
+   * When the <code>serve-private-resources-to-bots</code>
+   * configuration option is set to true, this function is called
+   * before the destruction of the WApplication if the resource was
+   * exposed as a private resource for a bot session. The returned
+   * resource will be added to the server's public resources, unless
+   * the returned value is a nullptr.
+   *
+   * The path at which the resource will be made available will be
+   * the value of the <code>bot-resources-path</code> option (specified
+   * in the configuration), followed by the botResourceId() (separated
+   * from the path by a slash).
+   *
+   * By default, this returns nullptr.
+   *
+   * \note If the session is a bot session, the url of this resource
+   *       will be the same as the url of the bot resource when this
+   *       function is called.
+   *
+   * \warning You are responsible for removing the resource once it is
+   *          no longer needed using WServer::removeResource() or
+   *          WServer::removeEntryPoint().
+   *
+   * \sa setBotResourceId(),
+   *     customBotResourceId(),
+   *     WSelfDeletingResource
+   */
+  virtual std::shared_ptr<WResource> botResource();
+
 protected:
   /*! \brief Prepares the resource for deletion.
    *
@@ -485,6 +573,14 @@ protected:
    */
   void beingDeleted();
 
+  /*! \brief Returns whether this resource has a custom bot resource ID.
+   *
+   * Returns whether the bot resource ID value was changed or not.
+   *
+   * \sa setBotResourceId(), botResourceId()
+   */
+  bool customBotResourceId() const { return customBotResourceId_; }
+
 private:
   struct UseLock {
     UseLock();
@@ -510,6 +606,8 @@ private:
   bool trackUploadProgress_;
   bool takesUpdateLock_;
   bool invalidAfterChanged_;
+  bool customBotResourceId_;
+  bool allowAutoRemoval_;
 
   std::vector<Http::ResponseContinuationPtr> continuations_;
 
@@ -525,6 +623,7 @@ private:
   std::string currentUrl_;
   std::string internalPath_;
   std::string botUrl_;
+  std::string botResourceId_;
   unsigned long version_;
 
   WApplication *app_; // associated app (for non-static resources)
