@@ -346,15 +346,25 @@ public:
    *
    * Use this method to remove an entry point or static resource.
    *
-   * In a multi-threaded environment, this may only be done when the
-   * server is not running.
-   *
    * \note If the entry point is a resource, the resource will not be deleted
    *
    * \sa addEntryPoint(), addResource()
    */
   WT_API void removeEntryPoint(const std::string& path);
 
+  /*! \brief Unbinds a resource from a fixed path.
+   *
+   * Use this method to remove a static resource.
+   *
+   * \note The resource object itself will not be destructed, and can
+           still be in use in other threads. However, any new requests
+           to its path will no longer be handled by the removed
+           resource.
+   *
+   * \sa addResource()
+   */
+  WT_API void removeResource(const WResource* resource);
+
 #ifndef WT_TARGET_JAVA
   /*! \brief Starts the server in the background.
    *
